SystemOrganization addCategory: #'Neo-CSV-Core'!

Object subclass: #NeoCSVReader
	instanceVariableNames: 'readStream separator stringStream fieldCount recordClass fieldAccessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-CSV-Core'!

!NeoCSVReader commentStamp: '<historical>' prior: 0!
I am NeoCSVReader.!

!NeoCSVReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/14/2012 12:31'!
on: readStream
	"Initialize on readStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."

	^ self new
		on: readStream;
		yourself! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 13:51'!
addField: accessor
	self addFieldAccessor: [ :object :string |
		object perform: accessor with: string ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 15:36'!
addField: accessor converter: converter
	self addFieldAccessor: [ :object :string |
		object perform: accessor with: (converter value: string) ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/16/2012 13:33'!
addFieldAccessor: block
	fieldAccessors 
		ifNil: [
			fieldAccessors := Array with: block ]
		ifNotNil: [
			fieldAccessors := fieldAccessors copyWith: block ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 13:51'!
addFields: accessors
	accessors do: [ :each |
		self addField: each ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 18:53'!
addFloatField: accessor
	self 
		addField: accessor 
		converter: [ :string | Float readFrom: string ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 18:53'!
addIntegerField: accessor
	self 
		addField: accessor 
		converter: [ :string | Integer readFrom: string ]! !

!NeoCSVReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/14/2012 12:34'!
atEnd
	^ readStream atEnd! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 12:32'!
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 11:33'!
fieldCount: anObject
	fieldCount := anObject! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 12:33'!
initialize
	super initialize.
	separator := $,! !

!NeoCSVReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2012 13:46'!
next
	^ (fieldAccessors isNil or: [ recordClass isNil ])
		ifTrue: [ self readNextRecordAsArray ] 
		ifFalse: [ self readNextRecordAsObject ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 12:32'!
on: aReadStream
	"Initialize on aReadStream, which should be a character stream that 
	implements #next, #peek, #atEnd and (optionally) #close."
	
	readStream := aReadStream! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:43'!
peekEndOfLine
	^ (readStream peek = Character lf) 
		or: [ readStream peek = Character cr ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:56'!
peekQuote
	^ readStream peek = $"! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:50'!
readEndOfLine
	readStream peek = Character lf
		ifTrue: [ 
			readStream next. 
			^ true ].
	readStream peek = Character cr
		ifTrue: [
			(readStream next; peek) = Character lf
				ifTrue: [ 
					readStream next ]. 
			^ true  ].
	^ false
! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 17:21'!
readEndOfQuotedField
	(self readQuote and: [ self peekQuote not ])
		ifTrue: [
			self readSeparator.
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 17:24'!
readField
	^ self peekQuote
		ifTrue: [
			self readQuotedField ]
		ifFalse: [
			self readUnquotedField ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 11:50'!
readFirstRecord 
	^ Array streamContents: [ :stream |
		[ self atEnd or: [ self readEndOfLine ] ]
			whileFalse: [ 
				stream nextPut: self readField ] ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:01'!
readNextRecord
	| record |
	record := Array new: fieldCount.
	1 to: fieldCount do: [ :each |
		record at: each put: self readField ].
	self assert: (self atEnd or: [ self readEndOfLine ]).
	^ record! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/16/2012 13:45'!
readNextRecordAsArray
	^ fieldCount 
		ifNil: [ | record |
			record := self readFirstRecord.
			fieldCount := record size.
			record ] 
		ifNotNil: [
			self readNextRecord ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/16/2012 13:45'!
readNextRecordAsObject
	| object |
	object := recordClass new.
	fieldAccessors do: [ :each |
		each value: object value: self readField ].
	self assert: (self atEnd or: [ self readEndOfLine ]).
	^ object ! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:56'!
readQuote
	self peekQuote
		ifTrue: [ 
			readStream next. 
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 17:23'!
readQuotedField
	self readQuote.
	^ self stringStreamContents: [ :stream |
		[ self atEnd or: [ self readEndOfQuotedField ] ]
			whileFalse: [
				stream nextPut: readStream next ] ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:16'!
readSeparator
	readStream peek = separator
		ifTrue: [ 
			readStream next. 
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 16:43'!
readUnquotedField
	^ self stringStreamContents: [ :stream |
		[ self atEnd or: [ self readSeparator or: [ self peekEndOfLine ] ] ]
			whileFalse: [ 
				stream nextPut: readStream next ] ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 13:47'!
recordClass: anObject
	recordClass := anObject! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 12:32'!
stringStreamContents: block
	"Like String streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	stringStream 
		ifNil: [ 
			stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents! !

!NeoCSVReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/14/2012 16:26'!
upToEnd 
	^ Array streamContents: [ :stream |
		[ self atEnd ]
			whileFalse: [ 
				stream nextPut: self next ] ]! !

Object subclass: #NeoCSVWriter
	instanceVariableNames: 'writeStream separator fieldWriter lineEnd fieldAccessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-CSV-Core'!

!NeoCSVWriter commentStamp: '<historical>' prior: 0!
I am NeoCSVWriter.
!

!NeoCSVWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/14/2012 09:47'!
on: writeStream
	"Initialize on writeStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	^ self new
		on: writeStream;
		yourself! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 13:34'!
addField: accessor
	self addFieldAccessor: [ :object |
		self writeField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 23:03'!
addFieldAccessor: block
	fieldAccessors 
		ifNil: [
			fieldAccessors := Array with: block ]
		ifNotNil: [
			fieldAccessors := fieldAccessors copyWith: block ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2012 13:34'!
addFields: accessors
	accessors do: [ :each |
		self addField: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:37'!
addObjectField: accessor
	self addFieldAccessor: [ :object |
		self writeObjectField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:38'!
addObjectFields: accessors
	accessors do: [ :each |
		self addObjectField: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:36'!
addQuotedField: accessor
	self addFieldAccessor: [ :object |
		self writeQuotedField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:39'!
addQuotedFields: accessors
	accessors do: [ :each |
		self addQuotedField: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:37'!
addRawField: accessor
	self addFieldAccessor: [ :object |
		self writeRawField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 22:38'!
addRawFields: accessors
	accessors do: [ :each |
		self addRawField: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 10:10'!
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 12:40'!
fieldWriter: symbol
	self assert: (#(quoted raw object) includes: symbol).
	fieldWriter := ('write', symbol capitalized, 'Field:') asSymbol
	! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 19:16'!
initialize 
	super initialize.
	lineEnd := String crlf.
	separator := $, .
	fieldWriter := #writeQuotedField: 
! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 19:16'!
lineEndConvention: symbol
	self assert: (#(cr lf crlf) includes: symbol).
	lineEnd := String perform: symbol! !

!NeoCSVWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2012 19:46'!
nextPut: anObject
	fieldAccessors 
		ifNil: [
			self writeFieldsUsingDo: anObject ]
		ifNotNil: [
			self writeFieldsUsingAccessors: anObject ].
	self writeEndOfLine ! !

!NeoCSVWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/14/2012 09:49'!
nextPutAll: collection
	collection do: [ :each | 
		self nextPut: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 09:48'!
on: aWriteStream
	"Initialize on aWriteStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	writeStream := aWriteStream
! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 10:11'!
separator: character
	self assert: character isCharacter.
	separator := character ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 19:16'!
writeEndOfLine
	writeStream nextPutAll: lineEnd ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:47'!
writeField: object
	self perform: fieldWriter with: object! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 22:37'!
writeFieldsUsingAccessors: anObject
	| first |
	first := true.
	fieldAccessors do: [ :each | | fieldValue | 
		first ifTrue: [ first := false ] ifFalse: [ self writeSeparator ].
		fieldValue := each value: anObject ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 19:45'!
writeFieldsUsingDo: anObject
	| first |
	first := true.
	anObject do: [ :each |
		first ifTrue: [ first := false ] ifFalse: [ self writeSeparator ].
		self writeField: each ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 10:51'!
writeObjectField: object
	object printOn: writeStream ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:37'!
writeQuotedField: object
	| string |
	string := object asString.
	writeStream nextPut: $".
	string do: [ :each |
		each = $" 
			ifTrue: [ writeStream nextPut: $"; nextPut: $" ]
			ifFalse: [ writeStream nextPut: each ] ].
	writeStream nextPut: $"! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:39'!
writeRawField: object
	writeStream nextPutAll: object asString! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 10:03'!
writeSeparator
	writeStream nextPut: separator ! !
